@@ -11,9 +11,11 @@ conda uninstall -y --force \
     zarr \
     cftime \
     rasterio \
+    packaging \
     pint \
     bottleneck \
     sparse \
+    h5netcdf \
     xarray
 # to limit the runtime of Upstream CI
 python -m pip install pytest-timeout
@@ -38,9 +40,11 @@ python -m pip install \
     git+https://github.com/dask/distributed \
     git+https://github.com/zarr-developers/zarr \
     git+https://github.com/Unidata/cftime \
-    git+https://github.com/mapbox/rasterio \
+    git+https://github.com/rasterio/rasterio \
+    git+https://github.com/pypa/packaging \
     git+https://github.com/hgrecco/pint \
     git+https://github.com/pydata/bottleneck \
     git+https://github.com/pydata/sparse \
     git+https://github.com/intake/filesystem_spec \
-    git+https://github.com/SciTools/nc-time-axis
+    git+https://github.com/SciTools/nc-time-axis \
+    git+https://github.com/h5netcdf/h5netcdf
