@@ -81,7 +81,7 @@ def _contains_obj_type(*, pat: Any, checker: Any) -> bool:
         return True
 
     # If it is not an object array it can't contain compiled re
-    if not getattr(pat, "dtype", "no") == np.object_:
+    if getattr(pat, "dtype", "no") != np.object_:
         return False
 
     return _apply_str_ufunc(func=checker, obj=pat).all()
@@ -95,7 +95,7 @@ def _contains_str_like(pat: Any) -> bool:
     if not hasattr(pat, "dtype"):
         return False
 
-    return pat.dtype.kind == "U" or pat.dtype.kind == "S"
+    return pat.dtype.kind in ["U", "S"]
 
 
 def _contains_compiled_re(pat: Any) -> bool:
@@ -114,7 +114,7 @@ def _apply_str_ufunc(
     obj: Any,
     dtype: Union[str, np.dtype, Type] = None,
     output_core_dims: Union[list, tuple] = ((),),
-    output_sizes: Mapping[Hashable, int] = None,
+    output_sizes: Mapping[Any, int] = None,
     func_args: Tuple = (),
     func_kwargs: Mapping = {},
 ) -> Any:
@@ -227,7 +227,7 @@ class StringAccessor:
         func: Callable,
         dtype: Union[str, np.dtype, Type] = None,
         output_core_dims: Union[list, tuple] = ((),),
-        output_sizes: Mapping[Hashable, int] = None,
+        output_sizes: Mapping[Any, int] = None,
         func_args: Tuple = (),
         func_kwargs: Mapping = {},
     ) -> Any:
@@ -270,11 +270,14 @@ class StringAccessor:
 
         if getattr(pat, "dtype", None) != np.object_:
             pat = self._stringify(pat)
-        func = lambda x: re.compile(x, flags=flags)
+
+        def func(x):
+            return re.compile(x, flags=flags)
+
         if isinstance(pat, np.ndarray):
             # apply_ufunc doesn't work for numpy arrays with output object dtypes
-            func = np.vectorize(func)
-            return func(pat)
+            func_ = np.vectorize(func)
+            return func_(pat)
         else:
             return _apply_str_ufunc(func=func, obj=pat, dtype=np.object_)
 
@@ -453,7 +456,7 @@ class StringAccessor:
             Strings or array-like of strings to concatenate elementwise with
             the current DataArray.
         sep : str or array-like of str, default: "".
-            Seperator to use between strings.
+            Separator to use between strings.
             It is broadcast in the same way as the other input strings.
             If array-like, its dimensions will be placed at the end of the output array dimensions.
 
@@ -536,7 +539,7 @@ class StringAccessor:
             Only one dimension is allowed at a time.
             Optional for 0D or 1D DataArrays, required for multidimensional DataArrays.
         sep : str or array-like, default: "".
-            Seperator to use between strings.
+            Separator to use between strings.
             It is broadcast in the same way as the other input strings.
             If array-like, its dimensions will be placed at the end of the output array dimensions.
 
